import React from 'react';
import {
  getDaysInMonth,
  startOfMonth,
  endOfMonth,
  getDay,
  getWeeksInMonth,
  addMonths,
  subMonths,
  format,
  getYear,
  getMonth,
  getDate,
  isSameDay,
} from 'date-fns';
import cx from 'classnames';
import './DatePicker.scss';

// TODO write function to get array of dates in month given the start and end date
// ? use array as values for table cells

export const DatePicker: React.FC = () => {
  const [date, setDate] = React.useState(new Date(Date.now()));
  const [selectedDate, setSelectedDate] = React.useState(date);
  const [showDatePicker, setShowDatePicker] = React.useState(false);

  console.log('days in month: ', getDaysInMonth(date));
  const firstOfMonth = startOfMonth(date);
  const lastOfMonth = endOfMonth(date);
  console.log('first date', firstOfMonth);
  const firstWeekdayOfMonth = getDay(firstOfMonth);
  const lastWeekdayOfMonth = getDay(lastOfMonth);

  const weeks = getWeeksInMonth(firstOfMonth);

  const pickerClasses = cx('date-picker__picker', {
    visible: showDatePicker,
  });

  return (
    <div className="date-picker">
      <p>Selected date: {format(selectedDate, 'dd/MM/yyyy')}</p>

      <button onClick={() => setShowDatePicker(!showDatePicker)}>
        Toggle DatePicker
      </button>

      <p>Should overlay this</p>

      <div className={pickerClasses}>
        <button onClick={() => setDate(subMonths(date, 1))}>
          Previous Month
        </button>
        <button onClick={() => setDate(addMonths(date, 1))}>Next Month</button>

        <button onClick={() => setDate(new Date(Date.now()))}>
          Go to today
        </button>

        <p>Month: {format(date, 'MMMM yyyy')}</p>

        <select
          value={date.getMonth()}
          onChange={event =>
            setDate(
              new Date(
                getYear(date),
                parseInt(event.target.value),
                getDate(date)
              )
            )
          }
        >
          <option value={0}>January</option>
          <option value={1}>February</option>
          <option value={2}>March</option>
          <option value={3}>April</option>
          <option value={4}>May</option>
          <option value={5}>June</option>
          <option value={6}>July</option>
          <option value={7}>August</option>
          <option value={8}>September</option>
          <option value={9}>October</option>
          <option value={10}>November</option>
          <option value={11}>December</option>
        </select>

        <select
          value={getYear(date)}
          onChange={event =>
            setDate(
              new Date(
                parseInt(event.target.value),
                getMonth(date),
                getDate(date)
              )
            )
          }
        >
          {Array.from(Array(100), (_, x) => 1950 + x).map(value => (
            <option key={value} value={value}>
              {value}
            </option>
          ))}
        </select>

        <table>
          <thead>
            <tr>
              <th>Sunday</th>
              <th>Monday</th>
              <th>Tuesday</th>
              <th>Wednesday</th>
              <th>Thursday</th>
              <th>Friday</th>
              <th>Saturday</th>
            </tr>
          </thead>
          <tbody>
            {[...Array(weeks).keys()].map((_, weekIndex) => (
              <tr className="date-picker__week" key={weekIndex}>
                {[...Array(7).keys()].map((_, dayIndex) => {
                  const weekMultiplier = 7 * weekIndex;

                  const dateValue = new Date(
                    getYear(date),
                    getMonth(date),
                    weekMultiplier + (dayIndex + 1) - firstWeekdayOfMonth
                  );

                  const isDisabled =
                    (weekIndex === 0 && dayIndex < firstWeekdayOfMonth) ||
                    (weekIndex === weeks - 1 && dayIndex > lastWeekdayOfMonth);

                  const dateClasses = cx('date-picker__day', {
                    disabled: isDisabled,
                    selected: isSameDay(dateValue, selectedDate),
                  });

                  return (
                    <td
                      className={dateClasses}
                      key={`${weekIndex}-${dayIndex}`}
                      tabIndex={isDisabled ? -1 : 0}
                      onClick={() => !isDisabled && setSelectedDate(dateValue)}
                    >
                      {isDisabled ? '' : dateValue.getDate()}
                    </td>
                  );
                })}
              </tr>
            ))}
          </tbody>
        </table>
      </div>
    </div>
  );
};
